package ui.view;

import javax.swing.JPanel;

import classes.TrackedDataSet;
import ui.controller.Control;
import ui.model.Model;

import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.SwingConstants;
import javax.swing.Timer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Hashtable;

public class StatisticGraphPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	// Model/Controller
	private Model model;
	private Control controller;

	// Components
	private StatisticGraph sGraph;
	private final JLabel graphNameLabel;
	private final JLabel maximumLabel = new JLabel("0");
	private JPanel topPanel = new JPanel();
	private JButton closeButton = new JButton("X");

	// Variables
	String graphName;
	private final JPanel legendPanel = new JPanel();
	private JPanel that;
	private Hashtable<String, StatisticGraphPanel> graphHashtable;
	private JPanel tempP; // for makeLegend

	/**
	 * Constructor.
	 * 
	 * @param mod
	 *            the Model
	 * @param cont
	 *            the Controller
	 */
	public StatisticGraphPanel(Model mod, Control cont, String name, Hashtable<String, StatisticGraphPanel> gHt) {
		super();
		this.model = mod;
		this.controller = cont;
		this.sGraph = new StatisticGraph(mod, cont);
		this.graphName = name;
		this.graphHashtable = gHt;
		setLayout(new BorderLayout(0, 0));

		// ******************** Component Propertys ***************//
		// Graph Name
		graphNameLabel = new JLabel(graphName);
		graphNameLabel.setHorizontalTextPosition(JLabel.CENTER);

		// Panel on top (Name and Close Button)
		topPanel.setLayout(new BorderLayout(0, 0));
		topPanel.add(graphNameLabel, BorderLayout.CENTER);
		topPanel.add(closeButton, BorderLayout.EAST);
		topPanel.setBorder(null);

		// Maximum Label
		maximumLabel.setVerticalAlignment(SwingConstants.TOP);
		maximumLabel.setMinimumSize(new Dimension(30, 10));

		// Legend Panel
		makeLegendPanel();

		// ******************** Component Listener ****************//

		that = this;
		closeButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				JPanel parent = (JPanel) that.getParent();
				for (int i = 0; i < parent.getComponentCount(); i++) {
					if (parent.getComponent(i).equals(that)) {
						parent.remove(parent.getComponent(i + 1));
						break;
					}
				}
				graphHashtable.remove(graphName);
				parent.remove(that);
				parent.updateUI();
			}
		});

		// ******************** add everything ********************//
		this.add(sGraph);
		this.add(topPanel, BorderLayout.NORTH);
		this.add(maximumLabel, BorderLayout.WEST);
		this.add(legendPanel, BorderLayout.SOUTH);

	}

	/**
	 * Adds the Set to the Graph.
	 * 
	 * @param set
	 */
	public void addObjec(TrackedDataSet set) {
		sGraph.addObject(set);
	}

	/**
	 * Set the Maximum Label
	 *
	 * @param max
	 */
	public void setMaximumLabel(double max) {
		maximumLabel.setText(Double.toString(max));
	}

	/**
	 * Get the name of the Graph.
	 * 
	 * @return the name of the Graph
	 */
	public String getGraphName() {
		return this.graphName;
	}

	/**
	 * Make the LegendPane.
	 */
	public void makeLegendPanel() {
		legendPanel.removeAll();
		for (TrackedDataSet set : sGraph.objects) {
			JLabel b = new JLabel(set.getCpsObject().getName());
			b.setBackground(Color.CYAN);
			b.setOpaque(true);
			tempP = new JPanel();
			tempP.add(b);
			b.addMouseListener(new MouseAdapter() {
				@Override
				public void mousePressed(MouseEvent e) {
					if (e.BUTTON3 == e.getButton()) {
						legendPanel.remove(tempP);
						that.updateUI();
					}
				}
			});
			legendPanel.add(tempP);
		}
		updateUI();
	}
}
